// Code generated by go tool dist; DO NOT EDIT.
// This is a bootstrap copy of C:\ci\go_1537370023945\work\go\src\cmd\cgo\zdefaultcc.go

//line C:\ci\go_1537370023945\work\go\src\cmd\cgo\zdefaultcc.go:1
// Code generated by go tool dist; DO NOT EDIT.

package main

const defaultPkgConfig = `pkg-config`
func defaultCC(goos, goarch string) string {
	switch goos+`/`+goarch {
	}
	return "gcc"
}
func defaultCXX(goos, goarch string) string {
	switch goos+`/`+goarch {
	}
	return "g++"
}
