// errorcheckdir -s

// Copyright 2016 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Verify error messages referring to multiple different
// packages with the same package name.

package ignored
